local harbour = ...

harbour:registerBehaviorTreeNode({
    Name = HarbourPrefix .. "_NODE",
    VariableList = {
		Var1 = "BEHAVIOR_TREE_DATA_AGENT",
		Var2 = "BEHAVIOR_TREE_DATA_WAIT",
		Var3 = "BEHAVIOR_TREE_DATA_LOCATION",
		Var4 = "BEHAVIOR_TREE_DATA_RESOURCE_TRANSPORT",
		Var5 = "BEHAVIOR_TREE_DATA_LOOP"
    },
    Init = function(self, instance)
		harbour:log("INIT!")
    end,
    Update = function(self, level, instance)
		harbour:log("UPDATE!")
		return self:updateFunc(level, instance)
    end,
    Finish = function(self, instance)
		harbour:log("FINISH!")
    end,
	updateFunc = function(self, level, instance)
		harbour:log("UPDATE FUNC!")
		harbour:log("Agent data: " .. tostring(self.Var1))
		harbour:log(tostring(self.Var1.Agent))
		harbour:log("Agent data owner: " .. tostring(self.Var1.Agent:getOwner()))

		local workplace = self.Var1.Agent:getOwner():getEnabledComponent("COMP_VILLAGER"):getJobInstance().Workplace
		-- No workplace found that matches this job
		if (workplace == nil or not workplace:isEnabled()) then
			self.Var1.Agent:setBehaviorMessage(foundation.localize("I_CANT_FIND_A_PLACE_TO_WORK"))
			return BEHAVIOR_TREE_NODE_RESULT.FALSE
		end
                local boatComponent = workplace:getOwner():findFirstObjectWithComponentDown(HarbourPrefix .. "_COMP_WATERLINE")
--                if (boatObject ~= nil) then
  --                  local boatComponent = boatObject:getComponent(HarbourPrefix .. "_COMP_WATERLINE")
                    if (boatComponent ~= nil) then
                        boatComponent:registerAssignedSailor(self.Var1.Agent:getOwner())
--		    end
                end

                harbour:log("Wait data: " .. tostring(self.Var2))
		harbour:log("Location data: " .. tostring(self.Var3))
		harbour:log("Location data destination: " .. tostring(self.Var3.DestinationPosition))
		harbour:log("Resource transport data: " .. tostring(self.Var4))
		harbour:log("Loop data: " .. tostring(self.Var5))
		return BEHAVIOR_TREE_NODE_RESULT.TRUE
	end,
})

harbour:registerBehaviorTree({
	Id = HarbourPrefix .. "_BEHAVIOR_TREE",
	VariableList = {
		{
			Name = "AgentData",
			DataType = "BEHAVIOR_TREE_DATA_AGENT",
			IsPublic = true,
			InitialValue = {} -- Optional if empty, leaves the default values (if any)
		},
		{
			Name = "DoJobTimer",
			DataType = "BEHAVIOR_TREE_DATA_WAIT",
			IsPublic = false,
			InitialValue = {
				TimeToWait = 0,
				Animation = AGENT_ANIMATION.GATHER,
				SetIdleAfterWait = false,
			}
		},
		{
			Name = "WorkplacePosition",
			DataType = "BEHAVIOR_TREE_DATA_LOCATION",
			IsPublic = false,
			InitialValue = {
				CanNavigateOnGround = true,
				CanNavigateOnWater = true,
				IsSetOrientationOnDestination = true,
			}
		},
		{
			Name = "TransportData",
			DataType = "BEHAVIOR_TREE_DATA_RESOURCE_TRANSPORT",
			IsPublic = false,
		},
		{
			Name = "WorkLoop",
			DataType = "BEHAVIOR_TREE_DATA_LOOP",
			IsPublic = true,
			InitialValue = {
				LoopCount = 10,
				Duration = 0,
				IsInfinite = true,
				IsDuration = false,
			}
		},
	},
	Root = {
		Name = "WorkingGlobalSequencer",
		Type = "SEQUENCER",
		Children = {
			{
				Name = "NeedFilledSelector",
				Type = "SELECTOR",
				Children = {
					{
						Name = "NeedFilledInverter",
						Type = "INVERTER",
						Child = {
							Name = "IsMyNeedFilled",
							Type = "IS_WORKPLACE_NEED_RESOURCE",
							AgentData = "AgentData",
						}
					},
					{
						Name = "FetchResourceSequencer",
						Type = "SEQUENCER",
						Children = {
							{
								Name = "FindResourceForWorkplace",
								Type = "FIND_RESOURCE_FOR_WORKPLACE",
								AgentData = "AgentData",
								TransportData = "TransportData",
							},
							{
								Name = "Transport",
								Type = "BEHAVIOR_TRANSPORT",
								AgentData = "AgentData",
								TransportData = "TransportData",
							},
						},
					},
				},
			},
			{
				Name = HarbourPrefix .. "_NODE_INSTANCE",
				Type = HarbourPrefix .. "_NODE",
				Var1 = "AgentData",
				Var2 = "DoJobTimer",
				Var3 = "WorkplacePosition",
				Var4 = "TransportData",
				Var5 = "WorkLoop",
			},
			{
				Name = "WorkLoopRepeater",
				Type = "REPEAT",
				RepeatData = "WorkLoop",
				Child = {
					Name = "WorkLoopSequencer",
					Type = "SEQUENCER",
					Children = {
						{
							Name = "SetupWork",
							Type = "SETUP_WORK",
							AgentData = "AgentData",
							TimeToWait = "DoJobTimer",
							WorkPosition = "WorkplacePosition",
						},
						{
							Name = "GoToResource",
							Type = "GO_TO",
							AgentData = "AgentData",
							Destination = "WorkplacePosition",
							BuildingPathType = "",
							AnimationData = "",
							AnimationSpeedMultiplier = "",
						},
						{
							Name = "ProduceResource",
							Type = "PRODUCE_RESOURCE",
							AgentData = "AgentData",
							TimeToWait = "DoJobTimer",
						},
						{
							Name = "GettingXp",
							Type = "GIVE_JOB_XP",
							AgentData = "AgentData",
							ShouldReceiveXp = "",
						}
					}
				}
			}
		}
	}
})